﻿using Syncfusion.Pdf;
using Syncfusion.Pdf.Graphics;
using Syncfusion.Windows.Diagram;
using SyncfusionDiagram.Models;
using SyncfusionDiagram.ViewModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace SyncfusionDiagram
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        MappingViewModel vm;
       // HierarchicalTreeLayout mtree;
        public MainWindow()
        {
            InitializeComponent();
            vm = new MappingViewModel();
            this.DataContext = vm;
            vm.LoopChanged += vm_LoopChanged;
            diagramView.NodeSelected += diagramView_NodeSelected;
            diagramView.NodeUnSelected += diagramView_NodeUnSelected;
            diagramView.Loaded += diagramView_Loaded;
         //   mtree = new HierarchicalTreeLayout(diagramModel, diagramView);
        }

        void diagramView_Loaded(object sender, RoutedEventArgs e)
        {
            overview.OverviewSourceAncestor = this;
        }

        void diagramView_NodeUnSelected(object sender, NodeRoutedEventArgs evtArgs)
        {
            foreach (var item in diagramView.SelectionList)  // gives all selected nodes list
            {

            }
        }

        void diagramView_NodeSelected(object sender, NodeRoutedEventArgs evtArgs)
        {
            foreach (var item in diagramView.SelectionList)  // gives all selected nodes list
            {

            }
        }

        void vm_LoopChanged(object sender, EventArgs e)
        {
            refershLayout(TreeOrientation.TopBottom);
        }

        private void treeViewAdv_SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            var nodes = diagramModel.Nodes;
            foreach (var item in nodes)
            {
                var v = ((Node)item).Content;
                if (v is Device)
                {
                    Device i = (Device)v;
                    if (i == (Device)treeViewAdv.SelectedItem)
                    {
                        Node node = diagramControl.FindByName(((Node)item).Name);
                        diagramView.BringIntoCenter(node);
                        break;
                    }
                }
            }
        }

        //Refreshing the Layout
        private void refershLayout(TreeOrientation treeorientation)
        {
            HierarchicalTreeLayout tree = new HierarchicalTreeLayout(diagramModel, diagramView);
            // DirectedTreeLayout tree = new DirectedTreeLayout(diagramModel, diagramView);
            diagramModel.Orientation = treeorientation;
            tree.RefreshLayout();
            (diagramView.Page as DiagramPage).InvalidateMeasure();
            (diagramView.Page as DiagramPage).InvalidateArrange();
        }

        private void ComboBoxItem_Selected_16(object sender, RoutedEventArgs e)
        {
            if (diagramView != null)
            {
                refershLayout(TreeOrientation.TopBottom);
            }
        }
        // Bottom to Top Orientation
        private void ComboBoxItem_Selected17(object sender, RoutedEventArgs e)
        {
            refershLayout(TreeOrientation.BottomTop);
        }

        //Left to Right Orientation
        private void ComboBoxItem_Selected_18(object sender, RoutedEventArgs e)
        {
            refershLayout(TreeOrientation.LeftRight);
        }

        //Right to Left Orientation
        private void ComboBoxItem_Selected_19(object sender, RoutedEventArgs e)
        {
            refershLayout(TreeOrientation.RightLeft);
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            diagramView.FitToPage();
        }


        private void CheckBox_Click(object sender, RoutedEventArgs e)
        {
            diagramView.IsPanEnabled = (bool)((CheckBox)e.Source).IsChecked;
        }

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            ExportToPDF();
        }

        private void Button_Click_2(object sender, RoutedEventArgs e)
        {
            diagramView.Print();
        }

        void ExportToPDF()
        {
            Microsoft.Win32.SaveFileDialog saveFileDialog = new Microsoft.Win32.SaveFileDialog();
            saveFileDialog.Filter = "PDF File|*.pdf";
            saveFileDialog.Title = "Save to a PDF file";
            saveFileDialog.ShowDialog();
            if (!string.IsNullOrEmpty(saveFileDialog.FileName))
            {
                float docWidth = (float)diagramView.Page.ActualWidth;
                float docHeight = (float)diagramView.Page.ActualHeight;
                string jpgFileName = "tempjpg.jpg";
                string xpsFileName = "tempXpS.xps";
                Syncfusion.XPS.XPSToPdfConverter xpsTopdfConverter = new Syncfusion.XPS.XPSToPdfConverter();
                Syncfusion.Pdf.Graphics.PdfUnitConvertor converter = new Syncfusion.Pdf.Graphics.PdfUnitConvertor();
                float pdfHeight = converter.ConvertUnits(docHeight, Syncfusion.Pdf.Graphics.PdfGraphicsUnit.Inch, Syncfusion.Pdf.Graphics.PdfGraphicsUnit.Point);
                float pdfWidth = converter.ConvertUnits(docWidth, Syncfusion.Pdf.Graphics.PdfGraphicsUnit.Inch, Syncfusion.Pdf.Graphics.PdfGraphicsUnit.Point);
                System.Drawing.SizeF pageSize = new System.Drawing.SizeF(pdfWidth, pdfHeight);
                diagramView.Save(jpgFileName, new Size(docWidth * 1.5, docHeight * 1.5), ImageStretch.Expand);
                Syncfusion.Pdf.PdfDocument pdfDocument = new Syncfusion.Pdf.PdfDocument();
                pdfDocument.PageSettings.Orientation = PdfPageOrientation.Landscape;
                pdfDocument.PageSettings.Margins.All = 0;
                PdfImage image = new PdfBitmap(jpgFileName);
                pdfDocument.PageSettings.Size = image.PhysicalDimension;
                Syncfusion.Pdf.PdfPage page = pdfDocument.Pages.Add();
                page.Graphics.DrawImage(image, 0, 0);
                pdfDocument.Save(saveFileDialog.FileName);
                pdfDocument.Close(true);
            }
        }

        private void HS_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (diagramModel != null)
            {
                HierarchicalTreeLayout mtree1 = new HierarchicalTreeLayout(diagramModel, diagramView);
                diagramModel.HorizontalSpacing = e.NewValue;
                mtree1.RefreshLayout();
            }
        }

        private void VS_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (diagramModel != null)
            {
                HierarchicalTreeLayout mtree1 = new HierarchicalTreeLayout(diagramModel, diagramView);
                diagramModel.VerticalSpacing = e.NewValue;
                mtree1.RefreshLayout();
            }
        }

        private void SN_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (diagramModel != null)
            {
                HierarchicalTreeLayout mtree1 = new HierarchicalTreeLayout(diagramModel, diagramView);
                diagramModel.SpaceBetweenSubTrees = e.NewValue;
                mtree1.RefreshLayout();
            }
        }

        private void Layout_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (diagramModel != null)
            {
                var v = (LayoutType)Enum.Parse(typeof(LayoutType), layout.SelectedValue.ToString());
                diagramModel.LayoutType = v;
                switch (v)
                {
                    case LayoutType.DirectedTreeLayout:
                        DirectedTreeLayout mtree1 = new DirectedTreeLayout(diagramModel, diagramView);
                        mtree1.RefreshLayout();
                        break;
                    case LayoutType.HierarchicalTreeLayout:
                        HierarchicalTreeLayout mtree2 = new HierarchicalTreeLayout(diagramModel, diagramView);
                        mtree2.RefreshLayout();
                        break;
                    case LayoutType.None:                                                
                        vm.ReloadLoopData();
                        diagramModel.LayoutType = LayoutType.None;
                        break;
                    case LayoutType.RadialTreeLayout:
                        RadialTreeLayout mtree3 = new RadialTreeLayout(diagramModel, diagramView);
                        mtree3.RefreshLayout();
                        break;
                }
            }
        }


    }
}
